//
//  Messagerie pour Graupner Hott V4
//  Module General Air
//  www.thyzoon.fr
//
// 2011-2013
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "Message.h"

// Trame message reponse
// 0x8D : Module General Air
uint8_t message_array[45] = {
  0x7c,0x8d,0x00,0xd0,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x7d,0x00};

//
// Emission de la trame
void GMessage::send(){ 
  // Calcul ChekSum 
  message_array[CHECKSUM_POS] = checksum(message_array);
  // Emission trame
  for(int i = 0; i < LENGTH; i++){
    Serial.write (message_array[i]);
    delay(TPS_EM);
    Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
  }
}

//
// Calcul CheckSum sur la trame a emettre
uint8_t GMessage::checksum (uint8_t *data)
{
  uint8_t sum = 0;
  // Calcul sur toute la trame sauf octet CheckSum
  for (uint8_t index = 0; index <= 43; index++)
  {
    sum = sum + data [index];                 
  } 
  return sum;
}

//////////////////////////////////////////////////
// Tension principale (resolution 0.1V)
void GMessage::setMainVoltage(float tension){
  uint16_t val = (uint16_t) (tension * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[30] = low; 
  message_array[31] = high;    
}
 
// Batterie 1 (resolution 0.1V)
void GMessage::setVoltage1(float volt){
  uint16_t val = (uint16_t) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[12] = low; 
  message_array[13] = high;    
}

// Batterie 2 (resolution 0.1V)
void GMessage::setVoltage2(float volt){
  uint16_t val = (uint16_t) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[14] = low; 
  message_array[15] = high;    
}

// Element Lipo n (resolution 0.2V)
void GMessage::setLipo(float volt, int lipo){
  message_array[5 + lipo] = (int) 100 * volt / 2;   
}

// Courant (A) : (resolution 0.1A)
void GMessage::setAmpere(float amp){
  uint16_t val = (uint16_t) (amp * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[28] = low; 
  message_array[29] = high;    
} 

// Courant (mA) : (resolution 10mA)
void GMessage::setMilliAmpere(uint16_t mA){
  mA = mA/10;
  uint8_t low =  (uint8_t) (mA&255);
  uint8_t high = (uint8_t)(mA >> 8);
  message_array[32] = low; 
  message_array[33] = high;    
}

// Altitude relative en metres : -500 / +9999 [m]
void GMessage::setAltitudeInMeters(uint16_t alti){
  uint8_t  p = (int) (alti + 500) / 256;
  uint8_t  l = alti -(p * 256 - 500);   
  message_array[24] = p;
  message_array[23] = l;
}

// Taux de montee en metre par seconde
// Non implementee
void GMessage::setClimbrate(float sr){
   //40 = 0 
  message_array[25] = 40; 
  //117 = 0 
  message_array[26] = 117;  
}

// Taux de montee en metre pour 3 secondes
// Non implementee
void GMessage::setClimbrate_M3(int srmmm){
  // 120 = 0
  message_array[27] = 120;
}

// Temperature : -20 a +235° C 
// 
void GMessage::setTemp(int temp, int capteur){
  if(temp < -20)
    temp = 20;
  if(temp > 234)
    temp = 235;
  if (capteur < 1)
    capteur = 1;
  if (capteur > 2)
    capteur = 2;      
  message_array[15 + capteur] = temp + 20;
  ;   
}

// Niveau reservoir : en %  
// 
void GMessage::setFuelPercent(uint8_t pourcent){
  if(pourcent > 100)
    pourcent = 100;
  message_array[18] = pourcent - 1;
}

// niveau reservoir en ml : 0 a 65535 ml
// 
void GMessage::setFuelMilliliters(uint16_t ml){
  uint8_t  h = (int) ml/256;
  uint8_t  u = ml-(h*256);   
  message_array[20] = h;
  message_array[19] = u;
}

// Rotation : 0 a 655350 tr/min
// 
void GMessage::setRPM(uint16_t rpm){
  uint8_t h = (uint8_t) (rpm/2560);
  message_array[22] = h;
  uint8_t  u = (uint8_t) (rpm-(h * 2560))/10;   
  message_array[21] = u;
} 

// N° version
// 
void GMessage::setVERSION(uint8_t ver){
  message_array[42] = ver;
} 


//
// Alarme sonore sur radio
//
void GMessage::alarme (uint8_t son){
  message_array[ALARM_BYTE_POS] = son;
}





